package org.rcisoft.bus.custinfo.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.bean.AdminProperties;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.bus.custinfo.dao.CustInfoRepository;
import org.rcisoft.bus.custinfo.entity.CustInfo;
import org.rcisoft.bus.custinfo.service.CustInfoService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2024年12月13日 上午9:58:24.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CustInfoServiceImpl extends ServiceImpl<CustInfoRepository,CustInfo>  implements CustInfoService {


    @Autowired
    protected AdminProperties adminProperties;


    /**
    * 保存 客户信息表
    * @param custInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CustInfo custInfo){
        //增加操作
        custInfo.setStoreId(CyUserUtil.getAuthenDept());
        int line = baseMapper.insert(custInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            custInfo.getBusinessId()+"的客户信息表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 客户信息表
    * @param custInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CustInfo custInfo){
        int line = baseMapper.realDelete(custInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            custInfo.getBusinessId()+"的客户信息表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 客户信息表
    * @param custInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(CustInfo custInfo){
        custInfo.setDeleted();
        int line = baseMapper.deleteById(custInfo);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                custInfo.getBusinessId()+"的客户信息表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 客户信息表
    * @param custInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CustInfo custInfo){
        int line = baseMapper.updateById(custInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ custInfo.getBusinessId()+"的客户信息表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 客户信息表
    * @param id
    * @return
    */
    @Override
    public CustInfo findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 客户信息表
    * @param custInfo
    * @return
    */
    @Override
    public IPage<CustInfo> findAllByPagination(CyPageInfo<CustInfo> paginationUtility,
        CustInfo custInfo){

        // 获取当前用户的角色ID
        String roleIds = CyUserUtil.getAuthenRoleIds();
        boolean adminKey = adminProperties.getAdmin().contains(CyUserUtil.getAuthenBusinessId());
        // 将角色ID字符串分割成数组，并去除每个ID前后的空白字符
        Set<String> userRoles = Arrays.stream(roleIds.split(","))
                .map(String::trim)
                .collect(Collectors.toSet());
        // 检查当前用户是否有管理员权限
        boolean isAdmin = userRoles.stream()
                .anyMatch(adminProperties.getAdminRole()::contains);
        // 根据是否是管理员调整查询条件
        if (!isAdmin || !adminKey) {
            custInfo.setStoreId(CyUserUtil.getAuthenDept());
        }

        return baseMapper.queryCustInfosPaged(paginationUtility,custInfo);

    }

    /**
    * 查询list 客户信息表
    * @param custInfo
    * @return
    */
    @Override
    public List<CustInfo> findAll(CustInfo custInfo){
        return baseMapper.queryCustInfos(custInfo);

    }

    /**
    * 导出客户信息表
    * @return
    */
    @Override
    public List<CustInfo> export(CustInfo custInfo) {
        List<CustInfo> custInfoList = baseMapper.queryCustInfos(custInfo);
        return custInfoList;
    }

}
