package org.rcisoft.bus.storegoodsapply.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.bean.AdminProperties;
import org.rcisoft.bus.storegoodsapply.dao.StoreGoodsApplyRepository;
import org.rcisoft.bus.storegoodsapply.dto.AddStoreGoodsApplyDTO;
import org.rcisoft.bus.storegoodsapply.entity.StoreGoodsApply;
import org.rcisoft.bus.storegoodsapply.service.StoreGoodsApplyService;
import org.rcisoft.bus.storegoodslog.entity.StoreGoodsLog;
import org.rcisoft.bus.storegoodslog.service.StoreGoodsLogService;
import org.rcisoft.bus.storegoodsstock.dao.StoreGoodsStockRepository;
import org.rcisoft.bus.storegoodsstock.entity.StoreGoodsStock;
import org.rcisoft.bus.storegoodsstock.service.StoreGoodsStockService;
import org.rcisoft.bus.wmsgoods.entity.WmsGoods;
import org.rcisoft.bus.wmsgoods.service.WmsGoodsService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by cy on 2024年5月29日 下午1:21:49.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class StoreGoodsApplyServiceImpl extends ServiceImpl<StoreGoodsApplyRepository, StoreGoodsApply> implements StoreGoodsApplyService {

    @Autowired
    private StoreGoodsStockService storeGoodsStockRepositoryImpl;
    @Autowired
    private StoreGoodsLogService storeGoodsLogServiceImpl;

    @Autowired
    private WmsGoodsService storeGoodsServiceImpl;

    @Autowired
    private StoreGoodsStockRepository storeGoodsStockRepository;


    @Autowired
    protected AdminProperties adminProperties;

    /**
     * 保存 商品申请表
     *
     * @param storeGoodsApply
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(StoreGoodsApply storeGoodsApply) {
        //增加操作
        //1.查库存表有没有对应的code  有则更新数量   没有则向库存表中插入
        //2.向日志表中添加对应的信息
        int line = baseMapper.insert(storeGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                storeGoodsApply.getBusinessId() + "的商品申请表信息");
        return new CyPersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel batchPersist(AddStoreGoodsApplyDTO addwmsGoodsApply) {
        // 入库
        StoreGoodsApply storeGoodsApply = new StoreGoodsApply();
        BeanUtils.copyProperties(addwmsGoodsApply, storeGoodsApply);
        storeGoodsApply.setIoType("0");
        storeGoodsApply.setDealStatus("0");
        storeGoodsApply.setStoreId(CyUserUtil.getAuthenDept());
        baseMapper.insert(storeGoodsApply);
        // 空
        if (CollectionUtil.isEmpty(addwmsGoodsApply.getBoxList())) {
            throw new CyServiceException(1001, "出库单为空");
        }
        List<StoreGoodsLog> logList = new ArrayList<>();
        StoreGoodsLog log = null;
        for (WmsGoods box : addwmsGoodsApply.getBoxList()) {
            log = new StoreGoodsLog();
            log.setGoodsName(box.getGoodsName());
            log.setGoodsCode(box.getGoodsCode());
            QueryWrapper<StoreGoodsStock> goodStockWrapper = new QueryWrapper<>();
            goodStockWrapper.eq("goods_id", box.getBusinessId());
            goodStockWrapper.eq("del_flag", "0");
            StoreGoodsStock goodsStock = storeGoodsStockRepository.selectOne(goodStockWrapper);
            if (goodsStock == null) {
                StoreGoodsStock newGoodsStock = new StoreGoodsStock();
                newGoodsStock.setGoodsId(box.getBusinessId());
                newGoodsStock.setGoodsCounts(box.getInCounts());
                storeGoodsStockRepositoryImpl.persist(newGoodsStock);
            } else {
                goodsStock.setGoodsCounts(goodsStock.getGoodsCounts() + box.getInCounts());
                storeGoodsStockRepositoryImpl.updateCounts(goodsStock);
            }
            //日志
            log.setApplyId(storeGoodsApply.getBusinessId());
            log.setGoodsId(box.getBusinessId());
            log.setGoodsCounts(box.getInCounts());
            log.setIoType("0");
            log.setUpdateCounts(goodsStock != null ? goodsStock.getGoodsCounts() : box.getInCounts());
            logList.add(log);
        }

        storeGoodsLogServiceImpl.batchPersist(logList);
        return new CyPersistModel(1);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel batchOutPersist(AddStoreGoodsApplyDTO addwmsGoodsApply) {
        // 1. 参数校验
        // 2. 插入 出库申请
        // 3. 循环
        //      3.1 库存不满足 => 异常
        //      3.2 库存满足 =>  减库存，日志
        //  4. 批量插入日志
        // 1. 集合空 判断
        if (CollectionUtil.isEmpty(addwmsGoodsApply.getBoxList())) {
            throw new CyServiceException(1001, "入库商品为空！");
        }
        // 出库
        StoreGoodsApply storeGoodsApplyOut = new StoreGoodsApply();
        BeanUtils.copyProperties(addwmsGoodsApply, storeGoodsApplyOut);
        storeGoodsApplyOut.setIoType("1");
        storeGoodsApplyOut.setDealStatus("1");
        storeGoodsApplyOut.setStoreId(CyUserUtil.getAuthenDept());
        baseMapper.insert(storeGoodsApplyOut);

        // 2. 遍历
        List<StoreGoodsLog> logList = new ArrayList<>();
        StoreGoodsLog log = null;
        for (WmsGoods box : addwmsGoodsApply.getBoxList()) {
            log = new StoreGoodsLog();
            log.setGoodsName(box.getGoodsName());
            log.setGoodsCode(box.getGoodsCode());

            StoreGoodsStock goodsStock = storeGoodsStockRepository.selectById(box.getBusinessId());
            // 库存不存在
            if (goodsStock == null) {
                throw new CyServiceException(1001, "库存数量不足");
            }
            //  库存不足
            if (box.getOutCounts() > goodsStock.getGoodsCounts()) {
                throw new CyServiceException(1001, "库存数量不足");
            }
            goodsStock.setGoodsCounts(goodsStock.getGoodsCounts() - box.getOutCounts());
            // 减库存
            storeGoodsStockRepositoryImpl.updateCounts(goodsStock);
            // 日志
            log.setApplyId(storeGoodsApplyOut.getBusinessId());
            log.setGoodsId(box.getGoodsId());
            log.setGoodsCounts(box.getOutCounts());
            log.setIoType("1");
            log.setUpdateCounts(goodsStock.getGoodsCounts());
            logList.add(log);
        }
        // 批量插入日志
        storeGoodsLogServiceImpl.batchPersist(logList);

        return new CyPersistModel(1);
    }

    @Override
    public StoreGoodsApply findByIdOut(int id) {
        StoreGoodsApply apply = baseMapper.selectById(id);
        if (apply == null) {
            throw new CyServiceException(1001, "申请单不存在!");
        }
        List<WmsGoods> goods = storeGoodsServiceImpl.findByStoreApplyId(apply);
        if(CollectionUtil.isNotEmpty(goods)){
            goods.forEach( x -> {
                x.setOutCounts(x.getInCounts());
                x.setInCounts(null);
            });
            apply.setStoreGoods(goods);
        }
        return apply;
    }

    @Override
    public String appCode() {
        // 获取当前时间
        Date date = new Date();
        // 设置日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        // 格式化日期
        String formattedDate = sdf.format(date);
        String codePrefix = "RX";
        int OrderCodeCount = baseMapper.getOrderCodeCount(codePrefix + formattedDate) + 1;
        String formattedCount = String.format("%03d", OrderCodeCount);
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 编号前缀
        String appCode = codePrefix + formattedDate + formattedCount;
        return appCode;
    }

    /**
     * 删除 商品申请表
     *
     * @param storeGoodsApply
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(StoreGoodsApply storeGoodsApply) {
        int line = baseMapper.realDelete(storeGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                storeGoodsApply.getBusinessId() + "的商品申请表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 商品申请表
     *
     * @param storeGoodsApply
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(StoreGoodsApply storeGoodsApply) {
        storeGoodsApply.setDeleted();
        int line = baseMapper.deleteById(storeGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                storeGoodsApply.getBusinessId() + "的商品申请表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 商品申请表
     *
     * @param storeGoodsApply
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(StoreGoodsApply storeGoodsApply) {
        int line = baseMapper.updateById(storeGoodsApply);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + storeGoodsApply.getBusinessId() + "的商品申请表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 商品申请表
     *
     * @param id
     * @return
     */
    @Override
    public StoreGoodsApply findById(int id) {
        StoreGoodsApply apply = baseMapper.selectById(id);
        if (apply == null) {
            throw new CyServiceException(1001, "入库申请不存在");
        }
        apply.setStoreGoods(storeGoodsServiceImpl.findByStoreApplyId(apply));
        return apply;
    }

    /**
     * 分页查询 商品申请表
     *
     * @param storeGoodsApply
     * @return
     */
    @Override
    public IPage<StoreGoodsApply> findAllByPagination(CyPageInfo<StoreGoodsApply> paginationUtility,
                                                      StoreGoodsApply storeGoodsApply) {
        // 获取当前用户的角色ID
        String roleIds = CyUserUtil.getAuthenRoleIds();
        boolean adminKey = adminProperties.getAdmin().contains(CyUserUtil.getAuthenBusinessId());
        // 将角色ID字符串分割成数组，并去除每个ID前后的空白字符
        Set<String> userRoles = Arrays.stream(roleIds.split(","))
                .map(String::trim)
                .collect(Collectors.toSet());
        // 检查当前用户是否有管理员权限
        boolean isAdmin = userRoles.stream()
                .anyMatch(adminProperties.getAdminRole()::contains);
        // 根据是否是管理员调整查询条件
        if (!isAdmin || !adminKey) {
            storeGoodsApply.setStoreId(CyUserUtil.getAuthenDept());
        }
        return baseMapper.queryWmsGoodsApplysPaged(paginationUtility, storeGoodsApply);

    }

    @Override
    public IPage<StoreGoodsApply> OutfindAllByPagination(CyPageInfo<StoreGoodsApply> paginationUtility,
                                                         StoreGoodsApply storeGoodsApply) {
        // 获取当前用户的角色ID
        String roleIds = CyUserUtil.getAuthenRoleIds();
        boolean adminKey = adminProperties.getAdmin().contains(CyUserUtil.getAuthenBusinessId());
        // 将角色ID字符串分割成数组，并去除每个ID前后的空白字符
        Set<String> userRoles = Arrays.stream(roleIds.split(","))
                .map(String::trim)
                .collect(Collectors.toSet());
        // 检查当前用户是否有管理员权限
        boolean isAdmin = userRoles.stream()
                .anyMatch(adminProperties.getAdminRole()::contains);
        // 根据是否是管理员调整查询条件
        if (!isAdmin || !adminKey) {
            storeGoodsApply.setStoreId(CyUserUtil.getAuthenDept());
        }
        return baseMapper.queryWmsGoodsApplysOutPaged(paginationUtility, storeGoodsApply);

    }

    /**
     * 查询list 商品申请表
     *
     * @param storeGoodsApply
     * @return
     */
    @Override
    public List<StoreGoodsApply> findAll(StoreGoodsApply storeGoodsApply) {
        return baseMapper.queryWmsGoodsApplys(storeGoodsApply);

    }

    /**
     * 导出商品申请表
     *
     * @return
     */
    @Override
    public List<StoreGoodsApply> export(StoreGoodsApply storeGoodsApply) {
        List<StoreGoodsApply> storeGoodsApplyList = baseMapper.queryWmsGoodsApplys(storeGoodsApply);
        return storeGoodsApplyList;
    }


    private String removeSymbol(String source) {
        String target = null;
        target = source.replaceAll("\\s*|\r|\n|\t", "");
        if (target.contains(" ")) {
            target.replaceAll(" ", "");
        }
        return target;
    }

    public Boolean isPositiveInteger(String figure) {
        // 如果输入为空，或者不是数字，则返回 false
        if (figure == null || figure.isEmpty()) {
            return false;
        }
        // 遍历字符串中的每个字符，检查是否都是数字
        for (char c : figure.toCharArray()) {
            if (!Character.isDigit(c)) {
                return false;
            }
        }
        // 将字符串转换为整数
        int num = Integer.parseInt(figure);
        // 如果大于 0 则为正整数，否则为非正整数
        return num > 0;
    }
}
