package org.rcisoft.bus.storegoodslog.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.bus.storegoodslog.dao.StoreGoodsLogRepository;
import org.rcisoft.bus.storegoodslog.entity.StoreGoodsLog;
import org.rcisoft.bus.storegoodslog.service.StoreGoodsLogService;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Created by cy on 2024年5月29日 下午1:21:49.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class StoreGoodsLogServiceImpl extends ServiceImpl<StoreGoodsLogRepository, StoreGoodsLog> implements StoreGoodsLogService {

    @Autowired
    private StoreGoodsLogService storeGoodsLogServiceImpl;

    /**
     * 保存 商品记录表
     *
     * @param storeGoodsLog
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(StoreGoodsLog storeGoodsLog) {
        //增加操作
        int line = baseMapper.insert(storeGoodsLog);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                storeGoodsLog.getBusinessId() + "的商品记录表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 商品记录表
     *
     * @param storeGoodsLog
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(StoreGoodsLog storeGoodsLog) {
        int line = baseMapper.realDelete(storeGoodsLog);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                storeGoodsLog.getBusinessId() + "的商品记录表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 商品记录表
     *
     * @param storeGoodsLog
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(StoreGoodsLog storeGoodsLog) {
        storeGoodsLog.setDeleted();
        int line = baseMapper.deleteById(storeGoodsLog);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                storeGoodsLog.getBusinessId() + "的商品记录表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 商品记录表
     *
     * @param storeGoodsLog
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(StoreGoodsLog storeGoodsLog) {
        int line = baseMapper.updateById(storeGoodsLog);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + storeGoodsLog.getBusinessId() + "的商品记录表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 商品记录表
     *
     * @param id
     * @return
     */
    @Override
    public StoreGoodsLog findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 商品记录表
     *
     * @param storeGoodsLog
     * @return
     */
    @Override
    public IPage<StoreGoodsLog> findAllByPagination(CyPageInfo<StoreGoodsLog> paginationUtility,
                                                    StoreGoodsLog storeGoodsLog) {
        return baseMapper.queryWmsGoodsLogsPaged(paginationUtility, storeGoodsLog);

    }

    /**
     * 查询list 商品记录表
     *
     * @param storeGoodsLog
     * @return
     */
    @Override
    public List<StoreGoodsLog> findAll(StoreGoodsLog storeGoodsLog) {
        return baseMapper.queryWmsGoodsLogs(storeGoodsLog);

    }

    /**
     * 导出商品记录表
     *
     * @return
     */
    @Override
    public List<StoreGoodsLog> export(StoreGoodsLog storeGoodsLog) {
        List<StoreGoodsLog> storeGoodsLogList = baseMapper.queryWmsGoodsLogs(storeGoodsLog);
        return storeGoodsLogList;
    }
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel batchPersist(List<StoreGoodsLog> addLogList) {
        baseMapper.batchPersist(addLogList);
        return new CyPersistModel(1);
    }
}
